using System.Security.Cryptography;
using System.Text;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.FileProviders;

namespace S01_ObjectCache.Services;

public class TextFileService {
    private const int SlidingExpirationInSeconds = 600; // 600 seconds = 10 minutes
    private readonly IMemoryCache cache;
    private readonly IFileProvider fileProvider;

    public TextFileService(IMemoryCache cache, IWebHostEnvironment env) {
        this.cache = cache;
        this.fileProvider = env.ContentRootFileProvider;
    }

    public async Task<string> GetTextFileContent(string fileName) {
        var fi = this.fileProvider.GetFileInfo(fileName);
        if (!fi.Exists) throw new FileNotFoundException("Requested file was not found.", fileName);

        // Get cache key based on file name
        var cacheKey = GetCacheKeyFromFileName(fi.PhysicalPath ?? throw new Exception("File does not have a physical path."));

        // Get value from cache or generate it and store it there
        var s = await cache.GetOrCreateAsync(cacheKey, async entry => {
            // Set file dependency
            entry.AddExpirationToken(this.fileProvider.Watch(fileName));
            // Set sliding expiration
            entry.SetSlidingExpiration(TimeSpan.FromSeconds(SlidingExpirationInSeconds));
            // Read the file content
            var text = await File.ReadAllTextAsync(fi.PhysicalPath);
            // Add curent time (for demo purposes)
            return string.Join(Environment.NewLine, DateTime.Now.ToString("s"), text);
        });

        // Return the value
        return s ?? string.Empty;
    }

    private static string GetCacheKeyFromFileName(string fileName) {
        var fnData = Encoding.UTF8.GetBytes(fileName);
        var fnHash = SHA256.HashData(fnData);
        return "file:" + string.Join(string.Empty, fnHash.Select(x => x.ToString("X2")));
    }
}